/**
 * ----------------------------------------------------------------------------------
 *
 * Copyright (C) 2011 runlevel3 GmbH Berlin
 *
 * ----------------------------------------------------------------------------------
 *
 * LICENSE:
 *
 * This file is part of USB-AD-Interface.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ----------------------------------------------------------------------------------
 *
 * Qt Creator 2.2.1 / Qt 4.7.3
 *
 * ----------------------------------------------------------------------------------
 *
 * @copyright 2011 runlevel3 GmbH
 * @license   http://www.gnu.org/licenses/gpl.txt GNU General Public License v3
 * @link      http://www.runlevel3.de runlevel3
 **/


#include <QTimer>
#include <QMessageBox>

#include "adconverter.h"
#include "mainwindow.h"
#include "ui_mainwindow.h"


extern "C" {
    #include <string.h>
    #include "usbhid.h"
}


QString window_title = "USB-AD-Interface";
QString about_notice = "USB-AD-Interface fuer Kraftmesszellen und Messbruecken\n\nCopyright 2011 runlevel3 GmbH Berlin\n";
QString version = "1.0";
QString version_date = "16.10.2011";

#define REFRESH_TYPE_ALL                    0
#define REFRESH_TYPE_OFFSET                 1
#define REFRESH_TYPE_GAIN                   2
#define REFRESH_TYPE_AD                     3
#define REFRESH_TYPE_AUTO                   4
#define REFRESH_TYPE_STARTUP_OFFSET         5
#define REFRESH_TYPE_LOOPS                  6
#define REFRESH_TYPE_MEAN                   7
#define REFRESH_TYPE_MEDIAN                 8
#define REFRESH_TYPE_VARIANCE               9
#define REFRESH_TYPE_STD_DEVIATION          10
#define REFRESH_TYPE_CALCULATE              11



//global variables
usbDevice_t *usbdev;
int usb_last_err = 0;


//function prototypes
void ui_refresh(Ui::MainWindow *ui, int refresh_type);



MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    this->setWindowTitle(window_title);
    this->setWindowIcon(QIcon(":/bitmaps/rl3.ico"));

    if ((usbdev = openUSBDevice()) == NULL) {
        QStringList msg;
        msg << usbVendor() << ": " << usbProduct() << ": open failed";
        ui->statusBar->showMessage(msg.join(""));
    }

    //init dialog elements
    QDoubleValidator *Validator_Vref = new QDoubleValidator(0, 5.0, 8, this);
    Validator_Vref->setNotation(QDoubleValidator::StandardNotation);
    ui->lineEdit_Vref->setValidator(Validator_Vref);
    QDoubleValidator *Validator_Faktor = new QDoubleValidator(-99999999, 99999999, 8, this);
    Validator_Faktor->setNotation(QDoubleValidator::StandardNotation);
    ui->lineEdit_Faktor->setValidator(Validator_Faktor);

    ADConverter0 = new ADConverter();
    ADTimer0 = new QTimer();
    connect(ADTimer0, SIGNAL(timeout()), this, SLOT(ADTimerUpdate()));
    RefreshTimer0 = new QTimer();
    connect(RefreshTimer0, SIGNAL(timeout()), this, SLOT(RefreshTimerUpdate()));

    ui->statusBar->addPermanentWidget(statusClearButton = new QPushButton("Clear"));
    connect(statusClearButton, SIGNAL(clicked()), this, SLOT(statusClearButton_clicked()));
    ui_refresh(ui, REFRESH_TYPE_ALL);
}

MainWindow::~MainWindow()
{
    if (usbdev) usbhidCloseDevice(usbdev);
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::on_actionQuit_triggered()
{
    qApp->quit();
}

void MainWindow::on_actionConnect_triggered()
{
    ui->statusBar->clearMessage();
    if (usbdev) usbhidCloseDevice(usbdev);
    
    if ((usbdev = openUSBDevice()) == NULL) {
        QStringList msg;
        msg << usbVendor() << ": " << usbProduct() << ": open failed";
        ui->statusBar->showMessage(msg.join(""));
    }
}

void MainWindow::statusClearButton_clicked()
{
    ui->statusBar->clearMessage();
}

void MainWindow::on_actionReboot_triggered()
{
    if (ADConverter0->setReboot() == -1) {
        ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
    }
    on_actionConnect_triggered();
}


void MainWindow::ui_refresh(Ui::MainWindow *ui, int refresh_type)
{
    int value;
    float float_value;
    QPalette lineEdit_Offset_Palette = ui->lineEdit_Offset->palette();
    QPalette lineEdit_Gain_Palette = ui->lineEdit_Gain->palette();

    switch(refresh_type) {
    case REFRESH_TYPE_ALL:
    case REFRESH_TYPE_OFFSET:
        if ((value = ADConverter0->getOffset()) == -1) {
            ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
        } else {
            ui->lineEdit_Offset->setText(QString::number(value, 16).rightJustified(8, '0', false).right(6).prepend('0'));
            lineEdit_Offset_Palette.setBrush(ui->lineEdit_Offset->backgroundRole(), Qt::white);
            ui->lineEdit_Offset->setPalette(lineEdit_Offset_Palette);
        }
        if (refresh_type == REFRESH_TYPE_OFFSET) break;
    case REFRESH_TYPE_GAIN:
        if ((value = ADConverter0->getGain()) == -1) {
            ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
        } else {
            ui->lineEdit_Gain->setText(QString::number(value, 16).rightJustified(8, '0', false).right(6).prepend('0'));
            lineEdit_Gain_Palette.setBrush(ui->lineEdit_Gain->backgroundRole(), Qt::white);
            ui->lineEdit_Gain->setPalette(lineEdit_Gain_Palette);
        }
        if (refresh_type == REFRESH_TYPE_GAIN) break;
    case REFRESH_TYPE_AD:
        if ((value = ADConverter0->getConvert(ui->spinBox_Channel->value(), ui->checkBox_Unipolar->isChecked() ? 1 : 0, ui->checkBox_Burnout->isChecked() ? 1 : 0)) == -1) {
            ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
        } else {
            ui->lineEdit_AD->setText(QString::number(value, 16).rightJustified(8, '0', false).right(6).prepend('0'));
        }
        if (refresh_type == REFRESH_TYPE_AD) break;
    case REFRESH_TYPE_AUTO:
        if ((value = ADConverter0->getAuto()) == -1) {
            ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
        } else {
            ui->checkBox_ADAuto->setChecked(value);
            ui->spinBox_ADInterval->setValue(value / 1000);
        }
        if (refresh_type == REFRESH_TYPE_AUTO) break;
    case REFRESH_TYPE_STARTUP_OFFSET:
        if ((value = ADConverter0->getStartupOffset()) == -1) {
            ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
        } else {
            ui->checkBox_StartupOffset->setChecked(value);
        }
        if (refresh_type == REFRESH_TYPE_STARTUP_OFFSET) break;
    case REFRESH_TYPE_LOOPS:
        if ((value = ADConverter0->getStatLoops()) == -1) {
            ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
        } else {
            ui->spinBox_Loops->setValue(value);
        }
        if (refresh_type == REFRESH_TYPE_LOOPS) break;
    case REFRESH_TYPE_MEAN:
        if ((value = ADConverter0->getMean()) == -1) {
            ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
        } else {
            ui->lineEdit_Mean->setText(QString::number(value, 16).rightJustified(8, '0', false).right(6).prepend('0'));
        }
        if (refresh_type == REFRESH_TYPE_MEAN) break;
    case REFRESH_TYPE_MEDIAN:
        if ((value = ADConverter0->getMedian()) == -1) {
            ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
        } else {
            ui->lineEdit_Median->setText(QString::number(value, 16).rightJustified(8, '0', false).right(6).prepend('0'));
        }
        if (refresh_type == REFRESH_TYPE_MEDIAN) break;
    case REFRESH_TYPE_VARIANCE:
        if ((float_value = ADConverter0->getVariance()) == -1) {
            ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
        } else {
            ui->lineEdit_Variance->setText(QString::number(float_value));
        }
        if (refresh_type == REFRESH_TYPE_VARIANCE) break;
    case REFRESH_TYPE_STD_DEVIATION:
        if ((float_value = ADConverter0->getStandardDeviation()) == -1) {
            ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
        } else {
            ui->lineEdit_StdDeviation->setText(QString::number(float_value));
        }
        if (refresh_type == REFRESH_TYPE_STD_DEVIATION) break;
    case REFRESH_TYPE_CALCULATE:
        int adinput;
        double advalue;

        adinput = ui->lineEdit_AD->text().toInt(NULL, 16);
        if (ui->radioButton_Mean->isChecked()) adinput = ui->lineEdit_Mean->text().toInt(NULL, 16);
        if (ui->radioButton_Median->isChecked()) adinput = ui->lineEdit_Median->text().toInt(NULL, 16);

        if (ui->checkBox_Unipolar->isChecked()) {
            advalue = adinput * ui->lineEdit_Vref->text().toDouble() / 16777216;
        } else {
            advalue = (adinput - 0x800000) * ui->lineEdit_Vref->text().toDouble() / 8388608;
        }
        if (ui->checkBox_ComputeGain->isChecked()) advalue /= ui->spinBox_Gain->value();

        ui->lineEdit_Vad->setText(QString::number(advalue, 'f', 8));
        advalue *= ui->lineEdit_Faktor->text().toDouble();
        ui->lineEdit_Vresult->setText(QString::number(advalue, 'f', 8));

        if (refresh_type == REFRESH_TYPE_CALCULATE) break;
    }
}


void MainWindow::on_lineEdit_Offset_returnPressed()
{
    ui->lineEdit_Offset->setText(ui->lineEdit_Offset->text().trimmed());
    if (ADConverter0->setOffset(ui->lineEdit_Offset->text().toInt(NULL, 16)) == -1) {
        ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
    }
    RefreshTimer0->start(2000);
}

void MainWindow::on_lineEdit_Offset_textChanged(QString )
{
    QPalette lineEdit_Offset_Palette = ui->lineEdit_Offset->palette();

    lineEdit_Offset_Palette.setBrush(ui->lineEdit_Offset->backgroundRole(), Qt::green);
    ui->lineEdit_Offset->setPalette(lineEdit_Offset_Palette);
}

void MainWindow::on_lineEdit_Gain_returnPressed()
{
    ui->lineEdit_Gain->setText(ui->lineEdit_Gain->text().trimmed());
    if (ADConverter0->setGain(ui->lineEdit_Gain->text().toInt(NULL, 16)) == -1) {
        ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
    }
    RefreshTimer0->start(2000);
}

void MainWindow::on_lineEdit_Gain_textChanged(QString )
{
    QPalette lineEdit_Gain_Palette = ui->lineEdit_Gain->palette();

    lineEdit_Gain_Palette.setBrush(ui->lineEdit_Gain->backgroundRole(), Qt::green);
    ui->lineEdit_Gain->setPalette(lineEdit_Gain_Palette);
}

void MainWindow::on_pushButton_OffsetSystemCalibration_clicked()
{
    if (ADConverter0->setOffsetSystemCalibration() == -1) {
        ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
    }
    RefreshTimer0->start(2000);
}

void MainWindow::on_pushButton_OffsetSelfCalibration_clicked()
{
    if (ADConverter0->setOffsetSelfCalibration() == -1) {
        ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
    }
    RefreshTimer0->start(2000);
}

void MainWindow::on_pushButton_GainSystemCalibration_clicked()
{
    if (ADConverter0->setGainSystemCalibration() == -1) {
        ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
    }
    RefreshTimer0->start(2000);
}

void MainWindow::on_pushButton_EepromSave_clicked()
{
    if (ADConverter0->setEeprom() == -1) {
        ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
    }
}

void MainWindow::on_pushButton_EepromReset_clicked()
{
    if (ADConverter0->resetEeprom() == -1) {
        ui->statusBar->showMessage(usbErrorMessage(usb_last_err));
    }
    RefreshTimer0->start(2000);
}

void MainWindow::ADTimerUpdate()
{
    //stop timer during update
    ADTimer0->stop();
    ui_refresh(ui, REFRESH_TYPE_AD);
    ui_refresh(ui, REFRESH_TYPE_MEAN);
    ui_refresh(ui, REFRESH_TYPE_MEDIAN);
    ui_refresh(ui, REFRESH_TYPE_VARIANCE);
    ui_refresh(ui, REFRESH_TYPE_STD_DEVIATION);
    ui_refresh(ui, REFRESH_TYPE_CALCULATE);
    if (ui->checkBox_ADAuto->isChecked()) {
        ADTimer0->start(ui->spinBox_ADInterval->value() * 1000);
    }
}

void MainWindow::RefreshTimerUpdate()
{
    RefreshTimer0->stop();
    ui_refresh(ui, REFRESH_TYPE_ALL);
}

void MainWindow::on_checkBox_ADAuto_toggled(bool checked)
{
    if (checked) {
        ADConverter0->setAuto(ui->spinBox_ADInterval->value() * 1000);
        ADTimer0->start(ui->spinBox_ADInterval->value() * 1000);
    } else {
        ADTimer0->stop();
        ADConverter0->setAuto(0);
    }
}

void MainWindow::on_spinBox_ADInterval_valueChanged(int i)
{
    ADConverter0->setAuto(i * 1000);
    ADTimer0->setInterval(i * 1000);
}

void MainWindow::on_pushButton_Convert_clicked()
{
    ADConverter0->getConvert(ui->spinBox_Channel->value(), ui->checkBox_Unipolar->isChecked() ? 1 : 0, ui->checkBox_Burnout->isChecked() ? 1 : 0);
    RefreshTimer0->start(2000);
}

void MainWindow::on_checkBox_StartupOffset_toggled(bool checked)
{
    if (checked) {
        ADConverter0->setStartupOffset(1);
    } else {
        ADConverter0->setStartupOffset(0);
    }
}

void MainWindow::on_spinBox_Loops_valueChanged(int i)
{
    ADConverter0->setStatLoops(i);
    ui_refresh(ui, REFRESH_TYPE_LOOPS);
}

void MainWindow::on_lineEdit_AD_textChanged(QString )
{
    ui_refresh(ui, REFRESH_TYPE_CALCULATE);
}

void MainWindow::on_lineEdit_Vref_editingFinished()
{
    ui_refresh(ui, REFRESH_TYPE_CALCULATE);
}

void MainWindow::on_checkBox_Unipolar_clicked()
{
    ui_refresh(ui, REFRESH_TYPE_CALCULATE);
}

void MainWindow::on_checkBox_ComputeGain_clicked()
{
    ui_refresh(ui, REFRESH_TYPE_CALCULATE);
}

void MainWindow::on_spinBox_Gain_valueChanged(int )
{
    ui_refresh(ui, REFRESH_TYPE_CALCULATE);
}

void MainWindow::on_actionAbout_triggered()
{
    QMessageBox::about(this, window_title, about_notice.append("\nVersion ").append(version).append(" vom ").append(version_date));
}
